/*
    PPOSXGlue_KeyUpDuringControlTracking.m

    Copyright 2013-2016 Josh Freeman
    http://www.twilightedge.com

    This file is part of PikoPixel for Mac OS X.
    PikoPixel is a graphical application for drawing & editing pixel-art images.

    PikoPixel is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License version 3,
    as published by the Free Software Foundation.

    PikoPixel is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//  On OS X Yosemite versions 10.10 - 10.10.2, keyboard events are intercepted by the system
// when a control is tracking the mouse; If a key is down before the user clicks on a control,
// and the key is released while the mouse is still down, the application won't receive
// notification.
//  As a workaround, several NSControl classes' mouseDown: methods and a pair of custom cell
// class' startTrackingAt:inView: methods are patched in order to manually post a delayed
// notification that is received as soon as tracking finishes - this allows objects that depend
// on the current keyboard state (PPDocumentWindowController, PPScreencastController) to know
// to recheck the keyboard's status.
//  The issue no longer appears as of 10.10.3.

#ifdef __APPLE__

#import <Cocoa/Cocoa.h>
#import "NSObject_PPUtilities.h"
#import "PPAppBootUtilities.h"
#import "PPLayerEnabledButtonCell.h"
#import "PPLayerOpacitySliderCell.h"
#import "NSNotificationCenter_PPUtilities.h"


@implementation NSObject (PPOSXGlue_KeyUpDuringControlTracking)

+ (void) ppOSXGlue_KeyUpDuringControlTracking_InstallPatches
{
    [NSSlider ppSwizzleInstanceMethodWithSelector: @selector(mouseDown:)
                forNSObjectInstanceMethodWithSelector: @selector(ppOSXPatch_MouseDown:)];

    [NSButton ppSwizzleInstanceMethodWithSelector: @selector(mouseDown:)
                forNSObjectInstanceMethodWithSelector: @selector(ppOSXPatch_MouseDown:)];

    [NSMatrix ppSwizzleInstanceMethodWithSelector: @selector(mouseDown:)
                forNSObjectInstanceMethodWithSelector: @selector(ppOSXPatch_MouseDown:)];

    [PPLayerEnabledButtonCell
        ppSwizzleInstanceMethodWithSelector: @selector(startTrackingAt:inView:)
        forNSObjectInstanceMethodWithSelector: @selector(ppOSXPatch_StartTrackingAt:inView:)];

    [PPLayerOpacitySliderCell
        ppSwizzleInstanceMethodWithSelector: @selector(startTrackingAt:inView:)
        forNSObjectInstanceMethodWithSelector: @selector(ppOSXPatch_StartTrackingAt:inView:)];
}

+ (void) load
{
    if (PP_RUNTIME_CHECK__RUNTIME_INTERCEPTS_KEYBOARD_EVENTS_WHEN_TRACKING_CONTROLS)
    {
        macroPerformNSObjectSelectorAfterAppLoads(
                                        ppOSXGlue_KeyUpDuringControlTracking_InstallPatches);
    }
}

- (void) ppOSXPatch_MouseDown: (NSEvent *) theEvent
{
    [self ppOSXPatch_MouseDown: theEvent];

    [NSNotificationCenter ppPostNotificationFromNewStackFrame_ControlDidEndTracking];
}

- (BOOL)  ppOSXPatch_StartTrackingAt: (NSPoint) startPoint inView: (NSView *) controlView
{
    [NSNotificationCenter ppPostNotificationFromNewStackFrame_ControlDidEndTracking];

    return [self ppOSXPatch_StartTrackingAt: startPoint inView: controlView];
}

@end

#endif  // __APPLE__
